@@ -1295,6 +1295,48 @@ menu_file_save_cb (BonoboUIComponent *uic,
 
 }
 
+static void
+menu_file_print_cb (BonoboUIComponent *uic,
+		   void *data,
+		   const char *path)
+{
+	CompEditor *editor = (CompEditor *) data;
+	CompEditorPrivate *priv = editor->priv;
+	ECalComponent *comp;
+	GList *l;
+	icalcomponent *icalcomp = e_cal_component_get_icalcomponent (priv->comp);
+	
+	comp = e_cal_component_new ();
+	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp));
+
+	for (l = priv->pages; l != NULL; l = l->next)
+		 comp_editor_page_fill_component (l->data, comp);
+	
+	print_comp (comp, priv->client, FALSE);
+
+	g_object_unref (comp);	
+}
+
+static void
+menu_file_print_preview_cb (BonoboUIComponent *uic,
+		   void *data,
+		   const char *path)
+{
+	CompEditor *editor = (CompEditor *) data;
+	CompEditorPrivate *priv = editor->priv;
+	ECalComponent *comp;
+	GList *l;
+	icalcomponent *icalcomp = e_cal_component_get_icalcomponent (priv->comp);
+	
+	comp = e_cal_component_new ();
+	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp));
+	for (l = priv->pages; l != NULL; l = l->next)
+		 comp_editor_page_fill_component (l->data, comp);	
+	print_comp (comp, priv->client, TRUE);
+
+	g_object_unref (comp);	
+}
+
 static void
 menu_file_close_cb (BonoboUIComponent *uic,
 		   void *data,
@@ -1416,6 +1458,8 @@ menu_help_cb (BonoboUIComponent *uic,
 static BonoboUIVerb verbs [] = {
 
 	BONOBO_UI_VERB ("FileSave", menu_file_save_cb),
+	BONOBO_UI_VERB ("CalendarPrint", menu_file_print_cb),
+	BONOBO_UI_VERB ("CalendarPrintPreview", menu_file_print_preview_cb),		
 	BONOBO_UI_VERB ("FileClose", menu_file_close_cb),
 
 	BONOBO_UI_VERB ("EditCopy", menu_edit_copy_cb),
@@ -2378,7 +2422,7 @@ fill_widgets (CompEditor *editor)
 		GSList *attachment_list = NULL;
 		e_cal_component_get_attachment_list (priv->comp, &attachment_list);
 		set_attachment_list (editor, attachment_list);
-		g_slist_foreach (attachment_list, g_free, NULL);
+		g_slist_foreach (attachment_list, (GFunc)g_free, NULL);
 		g_slist_free (attachment_list);
 	}	
 
