@@ -172,9 +172,8 @@ ecmm_is_cell_editable (ETableModel *etm, int col, int row)
 	g_return_val_if_fail (col >= 0 && col < E_CAL_MODEL_MEMOS_FIELD_LAST, FALSE);
 	g_return_val_if_fail (row >= -1 || (row >= 0 && row < e_table_model_row_count (etm)), FALSE);
 
-	if(col == E_CAL_MODEL_FIELD_SUMMARY)
-		retval = FALSE;
-	else if (col < E_CAL_MODEL_FIELD_LAST)
+
+	if (col < E_CAL_MODEL_FIELD_LAST)
 		retval = E_TABLE_MODEL_CLASS (e_cal_model_memos_parent_class)->is_cell_editable (etm, col, row);
 
 	return retval;
@@ -241,10 +240,17 @@ static void
 ecmm_fill_component_from_model (ECalModel *model, ECalModelComponent *comp_data,
 				ETableModel *source_model, gint row)
 {
+	icaltimetype start;
 	g_return_if_fail (E_IS_CAL_MODEL_MEMOS (model));
 	g_return_if_fail (comp_data != NULL);
 	g_return_if_fail (E_IS_TABLE_MODEL (source_model));
 
+	start = icalcomponent_get_dtstart (comp_data->icalcomp);
+	if (icaltime_compare_date_only (start, icaltime_null_time ()) == 0) {
+		start = icaltime_today ();
+		icalcomponent_set_dtstart (comp_data->icalcomp, start);
+	}
+
 }
 
 /**
