@@ -26,27 +26,34 @@
  * USA.
  */
 
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <gal/unicode/gunicode.h>
+
 #include "e-searching-tokenizer.h"
 
-#include "e-util/e-memory.h"
-#include "e-util/e-msgport.h"
+#include "libedataserver/e-memory.h"
+#include "libedataserver/e-msgport.h"
 
 #define d(x) 
 
 enum {
-	EST_MATCH_SIGNAL,
-	EST_LAST_SIGNAL
+	MATCH_SIGNAL,
+	LAST_SIGNAL
 };
-guint e_searching_tokenizer_signals[EST_LAST_SIGNAL] = { 0 };
 
-static void     e_searching_tokenizer_begin      (HTMLTokenizer *, gchar *);
+static guint signals[LAST_SIGNAL] = { 0, };
+
+static void     e_searching_tokenizer_begin      (HTMLTokenizer *, char *);
 static void     e_searching_tokenizer_end        (HTMLTokenizer *);
-static gchar   *e_searching_tokenizer_peek_token (HTMLTokenizer *);
-static gchar   *e_searching_tokenizer_next_token (HTMLTokenizer *);
+static char    *e_searching_tokenizer_peek_token (HTMLTokenizer *);
+static char    *e_searching_tokenizer_next_token (HTMLTokenizer *);
 static gboolean e_searching_tokenizer_has_more   (HTMLTokenizer *);
 
 static HTMLTokenizer *e_searching_tokenizer_clone (HTMLTokenizer *);
@@ -54,7 +61,7 @@ static HTMLTokenizer *e_searching_tokenizer_clone (HTMLTokenizer *);
 /*
   static const gchar *space_tags[] = { "br", NULL };*/
 
-GtkObjectClass *parent_class = NULL;
+static HTMLTokenizerClass *parent_class = NULL;
 
 /** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
 
@@ -111,7 +118,7 @@ shared_state_unref (SharedState *shared)
 /* Utility functions */
 
 /* This is faster and safer than glib2's utf8 abomination, but isn't exported from camel as yet */
-static __inline__ guint32
+static inline guint32
 camel_utf8_getc(const unsigned char **ptr)
 {
 	register unsigned char *p = (unsigned char *)*ptr;
@@ -241,7 +248,7 @@ dump_trie(struct _state *s, int d)
     http://www-sr.informatik.uni-tuebingen.de/~buehler/AC/AC.html
    for a neat demo */
 
-static __inline__ struct _match *
+static inline struct _match *
 g(struct _state *q, guint32 c)
 {
 	struct _match *m = q->matches;
@@ -257,7 +264,7 @@ build_trie(int nocase, int len, char **words)
 {
 	struct _state *q, *qt, *r;
 	char *word;
-	struct _match *m, *n;
+	struct _match *m, *n = NULL;
 	int i, depth;
 	guint32 c;
 	struct _trie *trie;
@@ -265,10 +272,10 @@ build_trie(int nocase, int len, char **words)
 	struct _state **state_depth;
 
 	trie = g_malloc(sizeof(*trie));
-	trie->root.matches = 0;
+	trie->root.matches = NULL;
 	trie->root.final = 0;
-	trie->root.fail = 0;
-	trie->root.next = 0;
+	trie->root.fail = NULL;
+	trie->root.next = NULL;
 
 	trie->state_chunks = e_memchunk_new(8, sizeof(struct _state));
 	trie->match_chunks = e_memchunk_new(8, sizeof(struct _match));
@@ -291,13 +298,13 @@ build_trie(int nocase, int len, char **words)
 			if (nocase)
 				c = g_unichar_tolower(c);
 			m = g(q, c);
-			if (m == 0) {
+			if (m == NULL) {
 				m = e_memchunk_alloc(trie->match_chunks);
 				m->ch = c;
 				m->next = q->matches;
 				q->matches = m;
 				q = m->match = e_memchunk_alloc(trie->state_chunks);
-				q->matches = 0;
+				q->matches = NULL;
 				q->fail = &trie->root;
 				q->final = 0;
 				if (state_depth_max < depth) {
@@ -335,9 +342,9 @@ build_trie(int nocase, int len, char **words)
 				c = m->ch;
 				qt = m->match;
 				r = q->fail;
-				while (r != 0 && (n = g(r, c)) == NULL)
+				while (r && (n = g(r, c)) == NULL)
 					r = r->fail;
-				if (r != 0) {
+				if (r != NULL) {
 					qt->fail = n->match;
 					if (qt->fail->final > qt->final)
 						qt->final = qt->fail->final;
@@ -564,7 +571,7 @@ output_match(struct _searcher *s, unsigned int start, unsigned int end)
 	endtoken = find_token(s, end);
 
 	if (starttoken == NULL || endtoken == NULL) {
-		printf("Cannot find match history for match %d-%d\n", start, end);
+		d(printf("Cannot find match history for match %d-%d\n", start, end));
 		return;
 	}
 
@@ -662,7 +669,7 @@ push_subpending(struct _searcher *s, int offstart, int offend)
 {
 	/* This is really an assertion, we just ignore the last pending match instead of crashing though */
 	if (s->submatchp >= s->words) {
-		printf("ERROR: submatch pending stack overflow\n");
+		d(printf("ERROR: submatch pending stack overflow\n"));
 		s->submatchp = s->words-1;
 	}
 
@@ -710,10 +717,10 @@ static char *
 searcher_next_token(struct _searcher *s)
 {
 	struct _token *token;
-	char *tok, *stok;
+	char *tok, *stok, *pre_tok;
 	struct _trie *t = s->t;
 	struct _state *q = s->state;
-	struct _match *m;
+	struct _match *m = NULL;
 	int offstart, offend;
 	guint32 c;
 
@@ -746,19 +753,19 @@ searcher_next_token(struct _searcher *s)
 		}
 
 		/* process whole token */
-		stok = tok;
+		pre_tok = stok = tok;
 		while ((c = camel_utf8_getc((const unsigned char **)&tok))) {
 			if ((s->flags & SEARCH_CASE) == 0)
 				c = g_unichar_tolower(c);
 			while (q && (m = g(q, c)) == NULL)
 				q = q->fail;
-			if (q == 0) {
+			if (q == NULL) {
 				/* mismatch ... reset state */
 				output_subpending(s);
 				q = &t->root;
-			} else {
+			} else if (m != NULL) {
 				/* keep track of previous offsets of utf8 chars, rotating buffer */
-				s->last[s->lastp] = s->offset + (tok-stok)-1;
+				s->last[s->lastp] = s->offset + (pre_tok-stok);
 				s->lastp = (s->lastp+1)&s->last_mask;
 
 				q = m->match;
@@ -791,9 +798,10 @@ searcher_next_token(struct _searcher *s)
 					}
 				}
 			}
+			pre_tok = tok;
 		}
 
-		s->offset += (tok-stok)-1;
+		s->offset += (pre_tok-stok);
 
 		flush_extra(s);
 	}
@@ -957,55 +965,54 @@ struct _ESearchingTokenizerPrivate {
 
 /** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
 
-/* shoudlnt' this be finalise? */
 static void
-e_searching_tokenizer_destroy (GtkObject *obj)
+e_searching_tokenizer_finalise (GObject *obj)
 {
 	ESearchingTokenizer *st = E_SEARCHING_TOKENIZER (obj);
 	struct _ESearchingTokenizerPrivate *p = st->priv;
-
+	
 	search_info_free (p->primary);
 	search_info_free (p->secondary);
 	if (p->engine)
 		searcher_free(p->engine);
-
+	
 	/* again wtf? 
 	shared_state_unref (st->priv->shared);
 	*/
-
-	g_free(p);
-
-	if (parent_class->destroy)
-		parent_class->destroy (obj);
+	
+	g_free (p);
+	
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		G_OBJECT_CLASS (parent_class)->finalize(obj);
 }
 
 static void
 e_searching_tokenizer_class_init (ESearchingTokenizerClass *klass)
 {
-	GtkObjectClass *obj_class = (GtkObjectClass *) klass;
+	GObjectClass *obj_class = (GObjectClass *) klass;
 	HTMLTokenizerClass *tok_class = HTML_TOKENIZER_CLASS (klass);
-
-	e_searching_tokenizer_signals[EST_MATCH_SIGNAL] =
-		gtk_signal_new ("match",
-				GTK_RUN_LAST,
-				obj_class->type,
-				GTK_SIGNAL_OFFSET (ESearchingTokenizerClass, match),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE,
-				0);
-	gtk_object_class_add_signals (obj_class, e_searching_tokenizer_signals, EST_LAST_SIGNAL);
-
-	obj_class->destroy = e_searching_tokenizer_destroy;
-
-	tok_class->begin      = e_searching_tokenizer_begin;
-	tok_class->end        = e_searching_tokenizer_end;
-
+	
+	parent_class = g_type_class_ref (HTML_TYPE_TOKENIZER);
+	
+	signals[MATCH_SIGNAL] =
+		g_signal_new ("match",
+			      E_TYPE_SEARCHING_TOKENIZER,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ESearchingTokenizerClass, match),
+			      NULL,
+			      NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+	
+	obj_class->finalize = e_searching_tokenizer_finalise;
+	
+	tok_class->begin = e_searching_tokenizer_begin;
+	tok_class->end = e_searching_tokenizer_end;
+	
 	tok_class->peek_token = e_searching_tokenizer_peek_token;
 	tok_class->next_token = e_searching_tokenizer_next_token;
-	tok_class->has_more   = e_searching_tokenizer_has_more;
-	tok_class->clone      = e_searching_tokenizer_clone;
-	
-	parent_class = gtk_type_class (HTML_TYPE_TOKENIZER);
+	tok_class->has_more = e_searching_tokenizer_has_more;
+	tok_class->clone = e_searching_tokenizer_clone;
 }
 
 static void
@@ -1024,31 +1031,32 @@ e_searching_tokenizer_init (ESearchingTokenizer *st)
 	search_info_set_colour(p->secondary, "purple");
 }
 
-GtkType
+GType
 e_searching_tokenizer_get_type (void)
 {
-	static GtkType e_searching_tokenizer_type = 0;
-
-	if (! e_searching_tokenizer_type) {
-		static GtkTypeInfo e_searching_tokenizer_info = {
-			"ESearchingTokenizer",
-			sizeof (ESearchingTokenizer),
+	static GType type = 0;
+	
+	if (!type) {
+		static const GTypeInfo info = {
 			sizeof (ESearchingTokenizerClass),
-			(GtkClassInitFunc) e_searching_tokenizer_class_init,
-			(GtkObjectInitFunc) e_searching_tokenizer_init,
 			NULL, NULL,
-			(GtkClassInitFunc) NULL
+			(GClassInitFunc) e_searching_tokenizer_class_init,
+			NULL, NULL,
+			sizeof (ESearchingTokenizer),
+			0,
+			(GInstanceInitFunc) e_searching_tokenizer_init,
 		};
-		e_searching_tokenizer_type = gtk_type_unique (HTML_TYPE_TOKENIZER,
-							     &e_searching_tokenizer_info);
+		
+		type = g_type_register_static (HTML_TYPE_TOKENIZER, "ESearchingTokenizer", &info, 0);
 	}
-	return e_searching_tokenizer_type;
+	
+	return type;
 }
 
 HTMLTokenizer *
 e_searching_tokenizer_new (void)
 {
-	return (HTMLTokenizer *) gtk_type_new (E_TYPE_SEARCHING_TOKENIZER);
+	return (HTMLTokenizer *) g_object_new (E_TYPE_SEARCHING_TOKENIZER, NULL);
 }
 
 /** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
@@ -1058,12 +1066,12 @@ e_searching_tokenizer_new (void)
 static char *get_token(HTMLTokenizer *t)
 {
 	HTMLTokenizerClass *klass = HTML_TOKENIZER_CLASS (parent_class);
-
+	
 	return klass->has_more(t) ? klass->next_token(t) : NULL;
 }
 
 static void
-e_searching_tokenizer_begin (HTMLTokenizer *t, gchar *content_type)
+e_searching_tokenizer_begin (HTMLTokenizer *t, char *content_type)
 {
 	ESearchingTokenizer *st = E_SEARCHING_TOKENIZER (t);
 	struct _ESearchingTokenizerPrivate *p = st->priv;
@@ -1105,7 +1113,7 @@ e_searching_tokenizer_end (HTMLTokenizer *t)
 	HTML_TOKENIZER_CLASS (parent_class)->end (t);
 }
 
-static gchar *
+static char *
 e_searching_tokenizer_peek_token (HTMLTokenizer *tok)
 {
 	ESearchingTokenizer *st = E_SEARCHING_TOKENIZER (tok);
@@ -1117,25 +1125,25 @@ e_searching_tokenizer_peek_token (HTMLTokenizer *tok)
 	return searcher_peek_token(st->priv->engine);
 }
 
-static gchar *
+static char *
 e_searching_tokenizer_next_token (HTMLTokenizer *tok)
 {
 	ESearchingTokenizer *st = E_SEARCHING_TOKENIZER (tok);
 	int oldmatched;
 	char *token;
-
+	
 	/* If no search is active, just use the default method. */
 	if (st->priv->engine == NULL)
 		return HTML_TOKENIZER_CLASS (parent_class)->next_token (tok);
-
+	
 	oldmatched = st->priv->engine->matchcount;
-
+	
 	token = searcher_next_token(st->priv->engine);
 
 	/* not sure if this has to be accurate or just say we had some matches */
 	if (oldmatched != st->priv->engine->matchcount)
-		gtk_signal_emit (GTK_OBJECT (st), e_searching_tokenizer_signals[EST_MATCH_SIGNAL]);
-
+		g_signal_emit (st, signals[MATCH_SIGNAL], 0);
+	
 	return token;
 }
 
@@ -1153,7 +1161,7 @@ static void
 matched (ESearchingTokenizer *st)
 {
 	/*++st->priv->match_count;*/
-	gtk_signal_emit (GTK_OBJECT (st), e_searching_tokenizer_signals[EST_MATCH_SIGNAL]);
+	g_signal_emit (st, signals[MATCH_SIGNAL], 0);
 }
 
 static HTMLTokenizer *
@@ -1161,25 +1169,22 @@ e_searching_tokenizer_clone (HTMLTokenizer *tok)
 {
 	ESearchingTokenizer *orig_st = E_SEARCHING_TOKENIZER (tok);
 	ESearchingTokenizer *new_st = E_SEARCHING_TOKENIZER (e_searching_tokenizer_new ());
-
+	
 	search_info_free(new_st->priv->primary);
 	search_info_free(new_st->priv->secondary);
-
+	
 	new_st->priv->primary = search_info_clone(orig_st->priv->primary);
 	new_st->priv->secondary = search_info_clone(orig_st->priv->secondary);
-
+	
 	/* what the fucking what???? */
 #if 0
 	shared_state_ref (orig_st->priv->shared);
 	shared_state_unref (new_st->priv->shared);
 	new_st->priv->shared = orig_st->priv->shared;
 #endif
-
-	gtk_signal_connect_object (GTK_OBJECT (new_st),
-				   "match",
-				   GTK_SIGNAL_FUNC (matched),
-				   GTK_OBJECT (orig_st));
-
+	
+	g_signal_connect_swapped (new_st, "match", G_CALLBACK(matched), orig_st);
+	
 	return HTML_TOKENIZER (new_st);
 }
 /* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
@@ -1188,7 +1193,7 @@ void
 e_searching_tokenizer_set_primary_search_string (ESearchingTokenizer *st, const gchar *search_str)
 {
 	g_return_if_fail (st && E_IS_SEARCHING_TOKENIZER (st));
-
+	
 	search_info_clear(st->priv->primary);
 	search_info_add_string(st->priv->primary, search_str);
 }
@@ -1197,7 +1202,7 @@ void
 e_searching_tokenizer_add_primary_search_string (ESearchingTokenizer *st, const gchar *search_str)
 {
 	g_return_if_fail (st && E_IS_SEARCHING_TOKENIZER (st));
-
+	
 	search_info_add_string(st->priv->primary, search_str);
 }
 
@@ -1205,7 +1210,7 @@ void
 e_searching_tokenizer_set_primary_case_sensitivity (ESearchingTokenizer *st, gboolean iscase)
 {
 	g_return_if_fail (st && E_IS_SEARCHING_TOKENIZER (st));
-
+	
 	search_info_set_flags(st->priv->primary, iscase?SEARCH_CASE:0, SEARCH_CASE);
 }
 
@@ -1213,7 +1218,7 @@ void
 e_searching_tokenizer_set_secondary_search_string (ESearchingTokenizer *st, const gchar *search_str)
 {
 	g_return_if_fail (st && E_IS_SEARCHING_TOKENIZER (st));
-
+	
 	search_info_clear(st->priv->secondary);
 	search_info_add_string(st->priv->secondary, search_str);
 }
@@ -1222,7 +1227,7 @@ void
 e_searching_tokenizer_add_secondary_search_string (ESearchingTokenizer *st, const gchar *search_str)
 {
 	g_return_if_fail (st && E_IS_SEARCHING_TOKENIZER (st));
-
+	
 	search_info_add_string(st->priv->secondary, search_str);
 }
 
@@ -1230,17 +1235,18 @@ void
 e_searching_tokenizer_set_secondary_case_sensitivity (ESearchingTokenizer *st, gboolean iscase)
 {
 	g_return_if_fail (st && E_IS_SEARCHING_TOKENIZER (st));
-
+	
 	search_info_set_flags(st->priv->secondary, iscase?SEARCH_CASE:0, SEARCH_CASE);
 }
 
+/* Note: only returns the primary search string count */
 gint
 e_searching_tokenizer_match_count (ESearchingTokenizer *st)
 {
-	g_return_val_if_fail (st && E_IS_SEARCHING_TOKENIZER (st), -1);
-
-	if (st->priv->engine)
+	g_return_val_if_fail (E_IS_SEARCHING_TOKENIZER (st), -1);
+	
+	if (st->priv->engine && st->priv->primary->strv->len)
 		return st->priv->engine->matchcount;
-
+	
 	return 0;
 }
