@@ -189,7 +189,7 @@ query_components (EComponentRegistry *registry)
 		GNOME_Evolution_Component iface;
 
 		id = info_list->_buffer[i].iid;
-		iface = bonobo_activation_activate_from_id (id, 0, NULL, &ev);
+		iface = bonobo_activation_activate_from_id ((char *)id, 0, NULL, &ev);
 		if (BONOBO_EX (&ev) || iface == CORBA_OBJECT_NIL) {
 			char *ex_text = bonobo_exception_get_text (&ev);
 
@@ -314,11 +314,11 @@ e_component_registry_peek_info (EComponentRegistry *registry,
 
 		switch (field) {
 		case ECR_FIELD_ID:
-			if (strcmp (info->id, key) == 0)
+			if (info->id && (strcmp (info->id, key) == 0))
 				return info;
 			break;
 		case ECR_FIELD_ALIAS:
-			if (strcmp (info->alias, key) == 0)
+			if (info->alias && (strcmp (info->alias, key) == 0))
 				return info;
 			break;
 		case ECR_FIELD_SCHEMA:
